@@ -1,98 +1,50 @@
-########################################################################
-# libs_curr_pm
-#
-# use $FindBin::Bin to search for 'lib' directories and use them.
-#
-# default action is to look for dir's named "lib" and silently use
-# the lib's without exporting anything. print turns on a short 
-# message with the abs_path results, export pushes out a variable
-# (default name is the base value), verbose turns on decision output
-# and print. export takes an optional argument with the name of a
-# variable to export.
-#
-# 
-########################################################################
-
-########################################################################
+#######################################################################
 # housekeeping
 ########################################################################
 
-package FindBin::libs   v2.12.01;
-
-use v5.14;
-use strict;
+package FindBin::libs   v3.0.0;
+use v5.28;
 
 use FindBin;
-
 use File::Basename;
 
 use Carp    qw( croak                   );
+use Cwd     qw( abs_path                );
 use Symbol  qw( qualify qualify_to_ref  );
 
 use File::Spec::Functions
 qw
 (
-    &splitpath
-    &splitdir
-    &catpath
-    &catdir
+    splitpath
+    splitdir
+    catpath
+    catdir
+    canonpath
 );
 
-BEGIN
-{
-    # however... there have been complaints of 
-    # places where abs_path does not work. 
-    #
-    # if abs_path fails on the working directory
-    # then replace it with rel2abs and live with 
-    # possibly slower, redundant directories.
-    #
-    # the abs_path '//' hack allows for testing 
-    # broken abs_path on primitive systems that
-    # cannot handle the rooted system being linked
-    # back to itself.
-
-    use Cwd qw( &abs_path &cwd );
-
-    unless( eval {abs_path '//';  abs_path cwd } )
-    {
-        # abs_path seems to be having problems,
-        # fix is to stub it out.
-        #
-        # undef avoids nastygram.
-
-        my $ref = qualify_to_ref 'abs_path', __PACKAGE__;
-
-        my $sub = File::Spec::Functions->can( 'rel2abs' );
-
-        undef &{ $ref };
-
-        *$ref = $sub
-    };
-}
-
 ########################################################################
 # package variables 
 ########################################################################
 
 my %defaultz = 
 (
-    base    => 'lib',
-    use     => undef,
-    blib    => undef,   # prefer ./blib at the first level
+    base    => 'lib'
+  , use     => undef
+  , blib    => undef    # prefer ./blib at the first level
 
-    subdir  => '',      # add this subdir also if found.
-    subonly => undef,   # leave out lib's, use only subdir.
-    export  => undef,   # push variable into caller's space.
-    append  => undef,   # push onto existing array (vs. overwrite)
-    verbose => undef,   # boolean: print inputs, results.
-    debug   => undef,   # boolean: set internal breakpoints.
+  , subdir  => '',      # add this subdir also if found.
+  , subonly => undef    # leave out lib's, use only subdir.
+  , export  => undef    # push variable into caller's space.
+  , append  => undef    # push onto existing array (vs. overwrite)
+  , verbose => undef    # boolean: print inputs, results.
+  , debug   => undef    # boolean: set internal breakpoints.
 
-    print   => 1,       # display the results
+  , print   => 1        # display the results
 
-    p5lib   => undef,   # prefix PERL5LIB with the results
+  , p5lib   => undef    # prefix PERL5LIB with the results
 
-    ignore => '/,/usr', # dir's to skip looking for ./lib
+  # dir's to skip looking for ./lib
+  , ignore => [ qw( / /usr ) ]
 );
 
 # only new directories are used, ignore pre-loads
@@ -102,8 +54,6 @@ my %found = ();
 
 # saves passing this between import and $handle_args.
 
-my %argz    = ();
-my $verbose = '';
 my $empty   = q{};
 
 ########################################################################
@@ -115,14 +65,50 @@ my $empty   = q{};
 # directory path for it on VMS. Fix is to unshift a leading
 # '' into @dirpath where the leading entry is true.
 
-my $find_libs
+my $find_dirs
 = sub
 {
-    my $base    = basename ( shift || $argz{ base } );
+$DB::single = 1;
 
-    my $subdir  = $argz{ subdir } || '';
+    my $argz    = shift;
+    my $base    = basename $argz->{ base };
+    my $subdir  
+    = $argz->{ subdir }
+    ? canonpath $argz->{ subdir }
+    : ''
+    ;
+    my $subonly = $argz->{ subonly } // '';
+    my %seen    = ();
+    my @libz    = ();
 
-    my $subonly = defined $argz{ subonly };
+    my $onepass = $argz->{ export } && $argz->{ scalar };
+    my $ignore  
+    = do
+    {
+        # ignoring nothing may be used with export,
+        # mainly for testing.
+
+        my $dirz    = $argz->{ ignore };
+        
+        if( @$dirz )
+        {
+            my $altz    
+            = join '|' =>
+            map
+            {
+                catdir $_, $base
+            }
+            @$dirz;
+
+            qr{^ (?: $altz ) $}x
+        }
+        else
+        {
+            qr{^$}
+        }
+    };
+
+    my $verbose = $argz->{ verbose };
 
     # for some reason, RH Enterprise V/4 has a 
     # trailing '/'; I havn't seen another copy of 
@@ -136,9 +122,9 @@ my $find_libs
     # after that splitpath can grab the directory 
     # portion for future use.
 
-    my ( $Bin ) = ( $argz{ Bin } =~ m{^ (.+) }xs );
+    my ( $Bin ) = ( $argz->{ Bin } =~ m{^ (.+) /? $}xs );
 
-    print STDERR "\nSearching $Bin for '$base'...\n"
+    print STDERR "\nSearching $Bin for '$base' ($subdir)...\n"
     if $verbose;
 
     my( $vol, $dir ) = splitpath $Bin, 1;
@@ -151,8 +137,6 @@ my $find_libs
 
     unshift @dirpath, '' if $dirpath[ 0 ];
 
-    my @libz    = ();
-
     PATH:
     for( 1 .. @dirpath )
     {
@@ -160,40 +144,53 @@ my $find_libs
         # volume only means something on DOS- & VMS-based
         # filesystems, and adding an empty basename on 
         # *nix is unnecessary.
-        #
-        # HAK ALERT: the poor slobs stuck on windog have an
-        # abs_path that croaks on missing directories. have
-        # to eval the check for subdir's. 
 
-        my $abs
-        = eval
+        my $dir = catpath $vol, ( catdir @dirpath, $base ), $empty;
+
+        $dir    =~ $ignore
+        and next;
+
+        -d $dir or next;
+
+        # regardless of how things are symlinked, the abspath
+        # check only puts each directory on the list once.
+
+        my @dirz
+        = do
         {
-            abs_path
-            catpath $vol, ( catdir @dirpath, $base ), $empty
-        }
-        || '';
-
-        my $sub
-        = $subdir
-        ? eval { abs_path ( catpath '', $abs, $subdir ) } || ''
-        : ''
-        ;
-
-        my @search = $subonly ? ( $sub ) : ( $abs, $sub );
-
-        for my $dir ( @search )
-        {
-            if( $dir && -d $dir && ! exists $found{ $dir } )
+            if( $subdir )
             {
-                $found{ $dir } = ();
+                my $sub = catpath $vol, ( catdir $dir, $subdir ), $empty;
 
-                push @libz, $dir;
-
-                last if $argz{ scalar };
+                if( -d $sub )
+                {
+                    $subonly
+                    ? ( $sub )
+                    : ( $sub, $dir )
+                }
+                else
+                {
+                    ()
+                }
+            }
+            else
+            {
+                ( $dir )
             }
         }
+        or next;
 
-        pop @dirpath
+        for my $dir ( @dirz )
+        {
+            $seen{ abs_path $dir } 
+            ||= push @libz, $dir;
+        }
+
+        last if $onepass;
+    }
+    continue
+    {
+        pop @dirpath;
     }
 
     # caller gets back the existing lib paths 
@@ -221,7 +218,7 @@ my $handle_args
     # anything after the module are options with arguments
     # assigned via '='.
 
-    %argz
+    my %argz
     = map
     {
         my $use_undef
@@ -273,6 +270,24 @@ my $handle_args
     : $FindBin::Bin
     ;
 
+    if( exists $argz{ ignore } )
+    {
+$DB::single = 1;
+        # ignore is different: it requires a bit of 
+        # compilation.
+        #
+        # ignore may be empty, mainly for testing.
+
+        if( my $dirz = $argz{ ignore } )
+        {
+            $argz{ ignore } = [ split ',' => $dirz ];
+        }
+        else
+        {
+            $argz{ ignore } = [];
+        }
+    }
+
     # now apply the defaults, then sanity check the result.
     # base is a special case since it always has to exist.
     #
@@ -296,50 +311,31 @@ my $handle_args
     and
     $argz{ export } //= $argz{ base };
 
-    $argz{ ignore } =
-    [
-        grep { $_ } split /\s*,\s*/, $argz{ ignore }
-    ];
+    $argz{ verbose } //= '';
 
-    $verbose = defined $argz{ verbose };
-
-    my $base = $argz{ base };
-
-    # now locate the libraries.
-    #
-    # %found contains the abs_path results for each directory to 
-    # avoid double-including directories.
-    #
-    # note: loop short-curcuts for the (usually) list.
-
-    %found = ();
-
-    for( @{ $argz{ ignore } } )
-    {
-        if( my $dir = eval { abs_path catdir $_, $base } )
-        {
-            if( -d $dir )
-            {
-                $found{ $dir } = 1;
-            }
-        }
-    }
+    %argz
 };
 
 sub import
 {
-    &$handle_args;
+    # map used here for -T checks. everything from find_dirs has
+    # already been through abs_path, not much more to check here.
 
-    my @libz = $find_libs->();
+    my %argz    = &$handle_args;
+    my @libz    
+    = map
+    {
+        my ( $path ) = m{ (.+) }x; 
 
-    # HAK ALERT: the regex does nothing for security,
-    # just dodges -T. putting this down here instead
-    # of inside find_libs allows people to use saner
-    # untainting plans via find_libs.
+        ( $path =~ /\S/ )
+        ? $path
+        : ()
+    }
+    $find_dirs->( \%argz )
+    ;
 
-    @libz   = map { m{ (.+) }xs } @libz;
-
-    my $caller = caller;
+    my $verbose = $argz{ verbose };
+    my $caller  = caller;
 
     if( $verbose || defined $argz{ print } )
     {
@@ -356,7 +352,7 @@ sub import
         # the caller is expecting to exist at runtime -- even
         # if they are empty/undef at the end of it.
 
-        my $ref     = qualify_to_ref $argz{ export }, $caller;
+        my $ref = qualify_to_ref $argz{ export }, $caller;
 
         if( $verbose )
         {
@@ -391,10 +387,10 @@ sub import
         }
     }
 
-    # no 'else', these are not exclusive
-
     if( @libz )
     {
+        # no 'else', these are not exclusive
+
         if( defined $argz{ p5lib } )
         {
             # stuff the lib's found at the front of $ENV{ PERL5LIB }
@@ -411,9 +407,6 @@ sub import
         {
             # this obviously won't work if lib ever depends 
             # on the caller's package.
-            #
-            # it does avoids issues with -T blowing up on the
-            # old eval technique.
 
             require lib;
 
@@ -425,9 +418,7 @@ sub import
 }
 
 # keep require happy
-
 1
-
 __END__
 
 =head1 NAME
@@ -437,9 +428,6 @@ directories based on $FindBin::Bin.
 
 =head1 SYNOPSIS
 
-This version of FindBin::libs is suitable for 
-Perl v5.10+.
-
     # search up $FindBin::Bin looking for ./lib directories
     # and "use lib" them.
 
@@ -932,7 +920,7 @@ may be limited to *NIX or at least less portable.
 
 PERL5LIB can be used to accomplish the same directory
 lookups as FindBin::libs.  The problem is PERL5LIB often
-contains absolte paths and does not automatically change
+contains absolute paths and does not automatically change
 depending on where tests are run. This can leave you 
 modifying a file, changing directory to see if it works
 with some other code and testing an unmodified version of 
@@ -974,7 +962,7 @@ in the executable path (or cwd) and uses that.
 
 The main problem here is that if the anchor ever changes
 (e.g., when moving code between projects or relocating 
-directories now that SVN supports it) the path often has
+directories) the path often has
 to change in multiple files. The regex also may have to
 support multiple platforms, or be broken into more complicated
 File::Spec code that probably looks pretty much like what
@@ -1039,8 +1027,12 @@ directory paths based on $FindBin::Bin.
 
 =item Older code.
 
-FindBin::libs_5_8.pm is installed if $^V indicates
-that the running perl is prior to v5.10.
+There are versions of the code for Perl from releases
+v5.8, 5.14, and 5.28. Makefile.PL will pick the most recent
+version it can based on $^V at Configure time. As of this
+writing Perl is at 5.32, which would pick the 5.28 release. 
+
+The version-specific code is in ./version/<perl-v-string.
 
 =back
 
@@ -1069,32 +1061,34 @@ bug, this is obviously a somewhat kludgy workaround
 and should be removed (with an added test for a 
 working version) once the File::Spec is fixed.
 
-=item 
-
-The hack for prior-to-5.12 versions of perl is 
-messy, but is the only I've found that works for
-the moment on *NIX, VMS, and MSW. I am not sure
-whether any of these systems are normally configured
-to share perl modules between versions. If the 
-moduels are not shared on multiple platforms then
-I can make this work by managing the installation
-rather than checking this every time at startup.
-
-For the moment, at least, this seems to work.
+Note that I have no way to test this on VMS at this
+point so the only thing I can think of is declaring
+the fix a feature.
 
 =back
 
+=head1 SEE ALSO
+
+=over 4
+
+=head2 Cwd
+
+A bug in Cwd was fixed with 3.73. That should, hopefully,
+fix the issue with FB::l croaking perl during testing.
+If it does not then I will have to modify the sanity check
+for using abs_path vs. rel2abs.
+
 =head1 AUTHOR
 
 Steven Lembark, Workhorse Computing <lembark@wrkhors.com>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2003-2014, Steven Lembark, Workhorse Computing.
-This code is released under the same terms as Perl-5.24
+Copyright (C) 2003-2022, Steven Lembark, Workhorse Computing.
+This code is released under the same terms as Perl-5.28
 or any later version of Perl.
 
 =head1 LICENSE
 
-This code is released under the same terms as Perl-5.24
+This code is released under the same terms as Perl-5.34
 or any later version of Perl.
